@@ -22,6 +22,7 @@
  */
 
 #include "config.h"
+#include "litest-config.h"
 
 #ifndef LITEST_H
 #define LITEST_H
@@ -34,6 +35,8 @@
 #include <libinput.h>
 #include <math.h>
 
+#include "check-double-macros.h"
+
 #include "libinput-util.h"
 #include "quirks.h"
 
@@ -264,6 +267,9 @@ enum litest_device_type {
 	LITEST_WACOM_CINTIQ_13HDT_PEN,
 	LITEST_WACOM_CINTIQ_13HDT_PAD,
 	LITEST_WACOM_CINTIQ_13HDT_FINGER,
+	LITEST_WACOM_CINTIQ_PRO16_PAD,
+	LITEST_WACOM_CINTIQ_PRO16_PEN,
+	LITEST_WACOM_CINTIQ_PRO16_FINGER,
 	LITEST_WACOM_HID4800_PEN,
 	LITEST_MOUSE_WHEEL_CLICK_COUNT,
 	LITEST_CALIBRATED_TOUCHSCREEN,
@@ -292,39 +298,40 @@ enum litest_device_type {
 };
 
 enum litest_device_feature {
-	LITEST_DEVICELESS = -2,
-	LITEST_DISABLE_DEVICE = -1,
-	LITEST_ANY = 0,
-	LITEST_TOUCHPAD = 1 << 0,
-	LITEST_CLICKPAD = 1 << 1,
-	LITEST_BUTTON = 1 << 2,
-	LITEST_KEYS = 1 << 3,
-	LITEST_RELATIVE = 1 << 4,
-	LITEST_WHEEL = 1 << 5,
-	LITEST_TOUCH = 1 << 6,
-	LITEST_SINGLE_TOUCH = 1 << 7,
-	LITEST_APPLE_CLICKPAD = 1 << 8,
-	LITEST_TOPBUTTONPAD = 1 << 9,
-	LITEST_SEMI_MT = 1 << 10,
-	LITEST_POINTINGSTICK = 1 << 11,
-	LITEST_FAKE_MT = 1 << 12,
-	LITEST_ABSOLUTE = 1 << 13,
-	LITEST_PROTOCOL_A = 1 << 14,
-	LITEST_HOVER = 1 << 15,
-	LITEST_ELLIPSE = 1 << 16,
-	LITEST_TABLET = 1 << 17,
-	LITEST_DISTANCE = 1 << 18,
-	LITEST_TOOL_SERIAL = 1 << 19,
-	LITEST_TILT = 1 << 20,
-	LITEST_TABLET_PAD = 1 << 21,
-	LITEST_RING = 1 << 22,
-	LITEST_STRIP = 1 << 23,
-	LITEST_TRACKBALL = 1 << 24,
-	LITEST_LEDS = 1 << 25,
-	LITEST_SWITCH = 1 << 26,
-	LITEST_IGNORED = 1 << 27,
-	LITEST_NO_DEBOUNCE = 1 << 28,
-	LITEST_TOOL_MOUSE = 1 << 29,
+	LITEST_DEVICELESS	= -2,
+	LITEST_DISABLE_DEVICE	= -1,
+	LITEST_ANY		= 0,
+	LITEST_TOUCHPAD		= bit(0),
+	LITEST_CLICKPAD		= bit(1),
+	LITEST_BUTTON		= bit(2),
+	LITEST_KEYS		= bit(3),
+	LITEST_RELATIVE		= bit(4),
+	LITEST_WHEEL		= bit(5),
+	LITEST_TOUCH		= bit(6),
+	LITEST_SINGLE_TOUCH	= bit(7),
+	LITEST_APPLE_CLICKPAD	= bit(8),
+	LITEST_TOPBUTTONPAD	= bit(9),
+	LITEST_SEMI_MT		= bit(10),
+	LITEST_POINTINGSTICK	= bit(11),
+	LITEST_FAKE_MT		= bit(12),
+	LITEST_ABSOLUTE		= bit(13),
+	LITEST_PROTOCOL_A	= bit(14),
+	LITEST_HOVER		= bit(15),
+	LITEST_ELLIPSE		= bit(16),
+	LITEST_TABLET		= bit(17),
+	LITEST_DISTANCE		= bit(18),
+	LITEST_TOOL_SERIAL	= bit(19),
+	LITEST_TILT		= bit(20),
+	LITEST_TABLET_PAD	= bit(21),
+	LITEST_RING		= bit(22),
+	LITEST_STRIP		= bit(23),
+	LITEST_TRACKBALL	= bit(24),
+	LITEST_LEDS		= bit(25),
+	LITEST_SWITCH		= bit(26),
+	LITEST_IGNORED		= bit(27),
+	LITEST_NO_DEBOUNCE	= bit(28),
+	LITEST_TOOL_MOUSE	= bit(29),
+	LITEST_DIRECT		= bit(30),
 };
 
 /* this is a semi-mt device, so we keep track of the touches that the tests
@@ -345,6 +352,7 @@ struct litest_semi_mt {
 };
 
 struct litest_device {
+	enum litest_device_type which;
 	struct libevdev *evdev;
 	struct libevdev_uinput *uinput;
 	struct libinput *libinput;
@@ -525,6 +533,15 @@ litest_touch_move_extended(struct litest_device *d,
 			   double y,
 			   struct axis_replacement *axes);
 
+void
+litest_touch_sequence(struct litest_device *d,
+		      unsigned int slot,
+		      double x1,
+		      double y1,
+		      double x2,
+		      double y2,
+		      int steps);
+
 void
 litest_touch_down(struct litest_device *d,
 		  unsigned int slot,
@@ -661,6 +678,9 @@ litest_wait_for_event_of_type(struct libinput *li, ...);
 void
 litest_drain_events(struct libinput *li);
 
+void
+litest_drain_events_of_type(struct libinput *li, ...);
+
 void
 litest_assert_event_type(struct libinput_event *event,
 			 enum libinput_event_type want);
@@ -1108,61 +1128,4 @@ litest_send_file(int sock, int fd)
 	return write(sock, buf, n);
 }
 
-#undef ck_assert_double_eq
-#undef ck_assert_double_ne
-#undef ck_assert_double_lt
-#undef ck_assert_double_le
-#undef ck_assert_double_gt
-#undef ck_assert_double_ge
-
-#define CK_DOUBLE_EQ_EPSILON 1E-3
-#define ck_assert_double_eq(X,Y)  \
-	do { \
-		double _ck_x = X; \
-		double _ck_y = Y; \
-		ck_assert_msg(fabs(_ck_x - _ck_y) < CK_DOUBLE_EQ_EPSILON, \
-			      "Assertion '" #X " == " #Y \
-			      "' failed: "#X"==%f, "#Y"==%f", \
-			      _ck_x, \
-			      _ck_y); \
-	} while (0)
-
-#define ck_assert_double_ne(X,Y)  \
-	do { \
-		double _ck_x = X; \
-		double _ck_y = Y; \
-		ck_assert_msg(fabs(_ck_x - _ck_y) > CK_DOUBLE_EQ_EPSILON, \
-			      "Assertion '" #X " != " #Y \
-			      "' failed: "#X"==%f, "#Y"==%f", \
-			      _ck_x, \
-			      _ck_y); \
-	} while (0)
-
-#define _ck_assert_double_eq(X, OP, Y)  \
-	do { \
-		double _ck_x = X; \
-		double _ck_y = Y; \
-		ck_assert_msg(_ck_x OP _ck_y || \
-			      fabs(_ck_x - _ck_y) < CK_DOUBLE_EQ_EPSILON, \
-			      "Assertion '" #X#OP#Y \
-			      "' failed: "#X"==%f, "#Y"==%f", \
-			      _ck_x, \
-			      _ck_y); \
-	} while (0)
-
-#define _ck_assert_double_ne(X, OP,Y) \
-	do { \
-		double _ck_x = X; \
-		double _ck_y = Y; \
-		ck_assert_msg(_ck_x OP _ck_y && \
-			      fabs(_ck_x - _ck_y) > CK_DOUBLE_EQ_EPSILON, \
-			      "Assertion '" #X#OP#Y \
-			      "' failed: "#X"==%f, "#Y"==%f", \
-			      _ck_x, \
-			      _ck_y); \
-	} while (0)
-#define ck_assert_double_lt(X, Y) _ck_assert_double_ne(X, <, Y)
-#define ck_assert_double_le(X, Y) _ck_assert_double_eq(X, <=, Y)
-#define ck_assert_double_gt(X, Y) _ck_assert_double_ne(X, >, Y)
-#define ck_assert_double_ge(X, Y) _ck_assert_double_eq(X, >=, Y)
 #endif /* LITEST_H */
